package search

import (
	"github.com/grafana/tempo/pkg/tempofb"
)

type DataCombiner struct{}

var staticCombiner = DataCombiner{}

func (*DataCombiner) Combine(_ string, searchData ...[]byte) ([]byte, bool, error) {

	if len(searchData) <= 0 {
		return nil, false, nil
	}

	if len(searchData) == 1 {
		return searchData[0], false, nil
	}

	// Squash all datas into 1
	data := tempofb.SearchEntryMutable{}
	kv := &tempofb.KeyValues{}   // buffer
	sd := &tempofb.SearchEntry{} // buffer
	for _, sb := range searchData {
		// we append zero-length entries to the WAL even when search is disabled. skipping to prevent unmarshalling and panik :)
		if len(sb) == 0 {
			continue
		}
		sd.Reset(sb)
		for i, ii := 0, sd.TagsLength(); i < ii; i++ {
			sd.Tags(kv, i)
			for j, jj := 0, kv.ValueLength(); j < jj; j++ {
				data.AddTag(string(kv.Key()), string(kv.Value(j)))
			}
		}

		data.SetStartTimeUnixNano(sd.StartTimeUnixNano())
		data.SetEndTimeUnixNano(sd.EndTimeUnixNano())
		data.TraceID = sd.Id()
	}

	return data.ToBytes(), true, nil
}
